using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using Microsoft.Win32;
using System.Reflection;
using EXGRIDLib;

namespace ExGridVirtualModeWrapper {
    /// <summary>
    /// UnboundHandler class
    /// </summary>
    [Guid("1578540e-0c96-414f-9f72-0c13d3d4a701")]
    [ProgId("ExGridVirtualModeWrapper.UnboundHandler")]
    [ClassInterface(ClassInterfaceType.AutoDual)]
    [ComVisible(true)]
    /// <summary>
    /// Unbound handler for the ExGrid component
    /// </summary>
    public class UnboundHandler : IUnboundHandler {
        private object _handler;
        [ComVisible(true)]
        public object Handler {
            get { return _handler; }
            set {
                _handler = value;
            }
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="handler"></param>
        public UnboundHandler() {
        }

        /// <summary>
        /// Returns the number of items to display
        /// </summary>
        /// <param name="o"></param>
        /// <returns></returns>
        public int get_ItemsCount(Object o) {
            int itemsCount = (int)_handler.GetType().InvokeMember("get_item_count", BindingFlags.InvokeMethod, null, _handler, null);
            return itemsCount;
        }

        /// <summary>
        /// Reads a specific item
        /// </summary>
        /// <param name="index"></param>
        /// <param name="source"></param>
        /// <param name="itemHandle"></param>
        public void ReadItem(int index, Object source, int itemHandle) {
            object[] args = new object[3];
            args[0] = index;
            args[1] = source;
            args[2] = itemHandle;
            _handler.GetType().InvokeMember("read_item", BindingFlags.InvokeMethod, null, _handler, args);
        }

        #region COM Registration
        //For more information on why the following two functions are necessary
        //http://www.codeproject.com/cs/miscctrl/exposingdotnetcontrols.asp
        //copyright Morgan Skinner, 2001
        [ComRegisterFunction()]
        public static void RegisterClass(string key) {
            // Strip off HKEY_CLASSES_ROOT\ from the passed key as I don't need it
            StringBuilder sb = new StringBuilder(key);
            sb.Replace(@"HKEY_CLASSES_ROOT\", "");

            // Open the CLSID\{guid} key for write access
            RegistryKey k = Registry.ClassesRoot.OpenSubKey(sb.ToString(), true);

            // And create the 'Control' key - this allows it to show up in 
            // the ActiveX control container 
            RegistryKey ctrl = k.CreateSubKey("Control");
            ctrl.Close();

            // Next create the CodeBase entry - needed if not string named and GACced.
            RegistryKey inprocServer32 = k.OpenSubKey("InprocServer32", true);
            inprocServer32.SetValue("CodeBase", Assembly.GetExecutingAssembly().CodeBase);
            inprocServer32.Close();

            // Finally close the main key
            k.Close();
        }

        [ComUnregisterFunction()]
        public static void UnregisterClass(string key) {
            StringBuilder sb = new StringBuilder(key);
            sb.Replace(@"HKEY_CLASSES_ROOT\", "");

            // Open HKCR\CLSID\{guid} for write access
            RegistryKey k = Registry.ClassesRoot.OpenSubKey(sb.ToString(), true);

            // Delete the 'Control' key, but don't throw an exception if it does not exist
            k.DeleteSubKey("Control", false);

            // Next open up InprocServer32
            RegistryKey inprocServer32 = k.OpenSubKey("InprocServer32", true);

            // And delete the CodeBase key, again not throwing if missing 
            k.DeleteSubKey("CodeBase", false);

            // Finally close the main key 
            k.Close();
        }
        #endregion
    }
}